@@ -559,7 +559,16 @@ void Options::printVersion()
   std::cout << "anytun";
 #endif
   std::cout << VERSION_STRING_0 << std::endl;
+
+#if defined(__clang__)
+  std::cout << VERSION_STRING_1 << ", using CLANG " << __clang_version__ << std::endl;
+#elif defined(__GNUC__)
+  std::cout << VERSION_STRING_1 << ", using GCC " << __GNUC__ << '.' << __GNUC_MINOR__
+            << '.' << __GNUC_PATCHLEVEL__ << std::endl;
+#else
   std::cout << VERSION_STRING_1 << std::endl;
+#endif
+
 }
 
 void Options::printUsage()
