@@ -65,7 +65,7 @@ SyncServer::~SyncServer()
   }
 }
 
-void SyncServer::onResolve(SyncTcpConnection::proto::resolver::iterator& it)
+void SyncServer::onResolve(SyncTcpConnection::proto::resolver::iterator it)
 {
   while(it != SyncTcpConnection::proto::resolver::iterator()) {
     SyncTcpConnection::proto::endpoint e = *it;
